/*

---Sessel-Steuerung für Sofa-Direkt Tonga---
--- ESP-Version                          ---

Michael Niemetz
2018-05-05
 */

#include <Arduino.h>
//#include "pins_arduino.h"
#include <JC_Button.h> //https://github.com/JChristensen/Button
#include <EEPROM.h>

#define RELAY_LUP 12   //Pin Relais Lehne runterfahren
#define RELAY_LDN 11   //Pin Relais Lehne hochfahren
#define RELAY_BUP 10   //Pin Relais Bank runterfahren
#define RELAY_BDN 9    //Pin Relais Bank hochfahren

#define BtnUp1Pin 8    //Pin Taster Lehne runter
#define BtnDn1Pin 7    //Pin Taster Lehne hoch
#define BtnUp2Pin 6    //Pin Taster Bank runter
#define BtnDn2Pin 5    //Pin Taster Bank hoch
#define BtnMemPin 4    //Pin Taster Speicher

#define PULLUP true
#define INVERT true
#define DEBOUNCE_MS 20
#define LONG_PRESS 1000    //Zeitdauer für einen langen Tastendruck

Button BtnUp1(BtnUp1Pin, PULLUP, INVERT, DEBOUNCE_MS); //Taste für Lehne runter
Button BtnDn1(BtnDn1Pin, PULLUP, INVERT, DEBOUNCE_MS); //Taste für Lehne hoch
Button BtnUp2(BtnUp2Pin, PULLUP, INVERT, DEBOUNCE_MS); //Taste für Bank runter
Button BtnDn2(BtnDn2Pin, PULLUP, INVERT, DEBOUNCE_MS); //Taste für Bank hoch
Button BtnMem(BtnMemPin, PULLUP, INVERT, DEBOUNCE_MS); //Taste zum Speichern/Setzen

enum {SET, STORE, RESET};
uint8_t BtnMemState;

int ms;
int startTime;  //Millis-Wert beim ersten Drücken der Taste
int duration;   //Dauer des Tastendrucks
int RelayTimeL = 0;
int RelayTimeB = 0;

int eeAddress = 0;
int StoredTimeL = 0;
int StoredTimeB = 0;

void setup(void) {

  pinMode(RELAY_LDN, OUTPUT);
  pinMode(RELAY_LUP, OUTPUT);
  pinMode(RELAY_BDN, OUTPUT);
  pinMode(RELAY_BUP, OUTPUT);
  pinMode(BtnUp1Pin, INPUT);
  pinMode(BtnDn1Pin, INPUT);
  pinMode(BtnUp2Pin, INPUT);
  pinMode(BtnDn2Pin, INPUT);
  pinMode(BtnMemPin, INPUT);

  digitalWrite(RELAY_LDN, HIGH);
  digitalWrite(RELAY_LUP, HIGH);
  digitalWrite(RELAY_BDN, HIGH);
  digitalWrite(RELAY_BUP, HIGH);

  BtnUp1.begin();
  BtnDn1.begin();
  BtnUp2.begin();
  BtnDn2.begin();
  BtnMem.begin();

  Serial.begin(9600);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  Serial.println("Ready!");
}

void loop(void) {
  ms = millis(); //aktuelle zeit Speichern
  BtnUp1.read();
  BtnDn1.read();
  BtnUp2.read();
  BtnDn2.read();
  BtnMem.read(); //Tastendruck auswerten

  buttonMem();
  buttonUpL();
  buttonDnL();
  buttonUpB();
  buttonDnB();
}

void buttonMem() //Auswertung langes und kurzes drücken zum setzen oder speichern
{
  switch (BtnMemState)
  {
    case SET:
    if (BtnMem.wasReleased())
    {
      setFromEprom();
      BtnMemState = RESET;
    }
    else if (BtnMem.pressedFor(LONG_PRESS))
    {
      BtnMemState = STORE;
    }
    break;

    case STORE:
    if (BtnMem.wasReleased())
    {
      storeToEprom();
      BtnMemState = RESET;
    }
    break;

    case RESET:
    if (BtnMem.wasReleased())
    {
      resetSitz();
      BtnMemState = SET;
    }
    break;
  }
}

void buttonUpL()
{
  if(BtnUp1.isPressed())
    {
      startTime = millis();
      while (BtnUp1.isPressed()) //Warten , solange Taste gedrückt wird
    {
      BtnUp1.read();
      digitalWrite(RELAY_LUP, LOW);
    }
    long duration = millis() - startTime;
    RelayTimeL += duration;
    Serial.print("ButtonUpLehne:");Serial.println(RelayTimeL);
    digitalWrite(RELAY_LUP, HIGH);
    BtnMemState = SET;
    }
}

void buttonUpB()
{
  if(BtnUp2.isPressed())
    {
      startTime = millis();
      while (BtnUp2.isPressed()) //Warten , solange Taste gedrückt wird
    {
      BtnUp2.read();
      digitalWrite(RELAY_BUP, LOW);
    }
    long duration = millis() - startTime;
    RelayTimeB += duration;
    Serial.print("ButtonUpBank:");Serial.println(RelayTimeB);
    digitalWrite(RELAY_BUP, HIGH);
    BtnMemState = SET;
    }
}

void buttonDnL()
{
  if(BtnDn1.isPressed())
    {
      startTime = millis();
      while (BtnDn1.isPressed()) //Warten , solange Taste gedrückt wird
    {
      BtnDn1.read();
      digitalWrite(RELAY_LDN, LOW);
    }
    long duration = millis() - startTime;
    RelayTimeL -= duration;
    if (RelayTimeL < 0) RelayTimeL = 0;
    Serial.print("ButtonDownLehne:");Serial.println(RelayTimeL);
    digitalWrite(RELAY_LDN, HIGH);
    BtnMemState = SET;
    }
}

void buttonDnB()
{
  if(BtnDn2.isPressed())
    {
      startTime = millis();
      while (BtnDn2.isPressed()) //Warten , solange Taste gedrückt wird
    {
      BtnDn2.read();
      digitalWrite(RELAY_BDN, LOW);
    }
    long duration = millis() - startTime;
    RelayTimeB -= duration;
    if (RelayTimeB < 0) RelayTimeB = 0;
    Serial.print("ButtonDownBank:");Serial.println(RelayTimeB);
    digitalWrite(RELAY_BDN, HIGH);
    BtnMemState = SET;
    }
}

void storeToEprom()
{
  eeAddress = 0;
  Serial.println("storeToEprom");
  if(EEPROM.get(eeAddress, StoredTimeL) != RelayTimeL){
    EEPROM.put(eeAddress, RelayTimeL);
    Serial.println("RelayTimeLehne gespeichert!"); Serial.println(eeAddress);
  }
  eeAddress += sizeof(int);
  if(EEPROM.get(eeAddress, StoredTimeB) != RelayTimeB){
    EEPROM.put(eeAddress, RelayTimeB);
    Serial.println("RelayTimeBank gespeichert!"); Serial.println(eeAddress);
  }
}

void setFromEprom()
{
  eeAddress = 0;
  Serial.println("setFromEprom");
  EEPROM.get(eeAddress,StoredTimeL);
  eeAddress += sizeof(int);
  EEPROM.get(eeAddress,StoredTimeB);
  Serial.print("StoredTimeLehne:"); Serial.println(StoredTimeL); Serial.print("RelayTimeLehne:"); Serial.println(RelayTimeL);
  Serial.print("StoredTimeBank:"); Serial.println(StoredTimeB); Serial.print("RelayTimeBank:"); Serial.println(RelayTimeB);
  if (StoredTimeL <= RelayTimeL)
  {
    Serial.println("StoredTimeL<=RelaytimeL");
    digitalWrite(RELAY_LDN, LOW);
    RelayTimeL = RelayTimeL - StoredTimeL;
    delay(RelayTimeL);
    RelayTimeL = StoredTimeL;
    digitalWrite(RELAY_LDN, HIGH);
    Serial.print("RelayTimeLehne:"); Serial.println(RelayTimeL);
  }
  else
  {
    Serial.println("StoredTimeL>RelaytimeL");
    digitalWrite(RELAY_LUP, LOW);
    RelayTimeL = StoredTimeL - RelayTimeL;
    delay(RelayTimeL);
    RelayTimeL = StoredTimeL;
    digitalWrite(RELAY_LUP, HIGH);
    Serial.print("RelayTimeLehne:"); Serial.println(RelayTimeL);
  }

  if (StoredTimeB <= RelayTimeB)
  {
    Serial.println("StoredTimeB<=RelaytimeB");
    digitalWrite(RELAY_BDN, LOW);
    RelayTimeB = RelayTimeB - StoredTimeB;
    delay(RelayTimeB);
    RelayTimeB = StoredTimeB;
    digitalWrite(RELAY_BDN, HIGH);
    Serial.print("RelayTimeBank:"); Serial.println(RelayTimeB);
  }
  else
  {
    Serial.println("StoredTimeB>RelaytimeB");
    digitalWrite(RELAY_BUP, LOW);
    RelayTimeB = StoredTimeB - RelayTimeB;
    delay(RelayTimeB);
    RelayTimeB = StoredTimeB;
    digitalWrite(RELAY_BUP, HIGH);
    Serial.print("RelayTimeBank:"); Serial.println(RelayTimeB);
  }

}

void resetSitz()
{
  Serial.println("ResetSitz");
  digitalWrite(RELAY_LDN, LOW);
  delay(RelayTimeL);
  digitalWrite(RELAY_LDN, HIGH);
  RelayTimeL = 0;
  digitalWrite(RELAY_BDN, LOW);
  delay(RelayTimeB);
  digitalWrite(RELAY_BDN, HIGH);
  RelayTimeB = 0;

}
